import QtQuick 1.1
import QtMobility.location 1.2
import "./js/RouteScript.js" as RouteScript

Item {
    id: pannableMap

    property alias zoomLevel: map.zoomLevel
    property alias mapCenter: map.center
    property bool satelliteMap: false
    property alias hereCenter: mapCircle.center
    property alias hereAccuracy: mapCircle.radius
    property alias mapElement: map

    /*
      Adds a coordinate to the bus route. Latitude and Longitude is passed and set
      on template coordinate. Creates bus stop image and place on given coordinate.
      Adds placed bus stops images on RouteScript.
    */
    function addRoutePoint(longitude, latitude,dir) {

            var coordinate = stopCoordinate.createObject(null);
            coordinate.longitude = longitude;
            coordinate.latitude = latitude;

            RouteScript.currentRoute = stopComponent.createObject(null);
            RouteScript.currentRoute.source = dir === 0 ? "./images/hereicon.png" : "./images/hereicon2.png";
            RouteScript.currentRoute.smooth = true;
            RouteScript.currentRoute.offset.x = -10;
            RouteScript.currentRoute.offset.y = -10;
            RouteScript.currentRoute.coordinate = coordinate;
            map.addMapObject(RouteScript.currentRoute);

            RouteScript.routes.push(RouteScript.currentRoute);
    }

    function addGalactico() {

        //Galerija Centrs
        var cdr = stopCoordinate.createObject(null);
        cdr.longitude = 24.113094;
        cdr.latitude = 56.94756;

        var obj1 = galacticoComponent.createObject(null);
        obj1.source = "./images/Galactico.png";
        obj1.smooth = true;
        obj1.offset.x = -25;
        obj1.offset.y = -25;
        obj1.coordinate = cdr;

        var obj1tt = galacticoComponent.createObject(null);
        obj1tt.source = "./images/galerija.png";
        obj1tt.offset.x = -90;
        obj1tt.offset.y = -140;
        obj1tt.coordinate = cdr;
        map.addMapObject(obj1);
        map.addMapObject(obj1tt);


        //Alfa
        var cdr2 = stopCoordinate.createObject(null);
        cdr2.longitude = 24.203251;
        cdr2.latitude = 56.983178;

        var obj2 = galacticoComponent.createObject(null);
        obj2.source = "./images/Galactico.png";
        obj2.smooth = true;
        obj2.offset.x = -25;
        obj2.offset.y = -25;
        obj2.coordinate = cdr2;

        var obj2tt = galacticoComponent.createObject(null);
        obj2tt.source = "./images/alfa.png";
        obj2tt.offset.x = -90;
        obj2tt.offset.y = -140;
        obj2tt.coordinate = cdr2;
        map.addMapObject(obj2);
        map.addMapObject(obj2tt);

        //Dole
        var cdr3 = stopCoordinate.createObject(null);
        cdr3.longitude = 24.19073;
        cdr3.latitude = 56.905832;

        var obj3 = galacticoComponent.createObject(null);
        obj3.source = "./images/Galactico.png";
        obj3.smooth = true;
        obj3.offset.x = -25;
        obj3.offset.y = -25;
        obj3.coordinate = cdr3;

        var obj3tt = galacticoComponent.createObject(null);
        obj3tt.source = "./images/dole.png";
        obj3tt.offset.x = -90;
        obj3tt.offset.y = -140;
        obj3tt.coordinate = cdr3;
        map.addMapObject(obj3);
        map.addMapObject(obj3tt);

        //Mols
        var cdr4 = stopCoordinate.createObject(null);
        cdr4.longitude = 24.14399;
        cdr4.latitude = 56.934483;

        var obj4 = galacticoComponent.createObject(null);
        obj4.source = "./images/Galactico.png";
        obj4.smooth = true;
        obj4.offset.x = -25;
        obj4.offset.y = -25;
        obj4.coordinate = cdr4;

        var obj4tt = galacticoComponent.createObject(null);
        obj4tt.source = "./images/mols.png";
        obj4tt.offset.x = -90;
        obj4tt.offset.y = -140;
        obj4tt.coordinate = cdr4;
        map.addMapObject(obj4);
        map.addMapObject(obj4tt);

        //Origo
        var cdr5 = stopCoordinate.createObject(null);
        cdr5.longitude = 24.121331;
        cdr5.latitude = 56.947511;

        var obj5 = galacticoComponent.createObject(null);
        obj5.source = "./images/Galactico.png";
        obj5.smooth = true;
        obj5.offset.x = -25;
        obj5.offset.y = -25;
        obj5.coordinate = cdr5;

        var obj5tt = galacticoComponent.createObject(null);
        obj5tt.source = "./images/origo.png";
        obj5tt.offset.x = -90;
        obj5tt.offset.y = -140;
        obj5tt.coordinate = cdr5;
        map.addMapObject(obj5);
        map.addMapObject(obj5tt);
    }

    /*
      Clears the bus route.
    */
    function clearRoute() {
        while (RouteScript.routes.length > 0) {
            var MapImage = RouteScript.routes.pop();
            map.removeMapObject(MapImage);
            MapImage.destroy();
        }
        RouteScript.route = null;
    }


    /*!
      Pans the map to the given coordinate.
    */
    function panToCoordinate(lat,lng) {
        panAnimation.latitude = lat;
        panAnimation.longitude = lng;
        panAnimation.restart();
    }


    Component {
        id: galacticoComponent
        MapImage {
        }
    }

    // Component for transport stop images
    Component {
        id: stopComponent

        MapImage { }
    }

    Component {
        id: stopCoordinate

        Coordinate { }
    }


    Map {
        id: map

        anchors {
            fill: parent
        }

        plugin : Plugin {
                name : "nokia"
                parameters: [
                    PluginParameter { name: "app_id"; value: "TmVnwnp3hs0LSH4-fwV8" },
                    PluginParameter { name: "token"; value: "5xApktvP4C3WG2Cnu3tB4w" }
                ]
        }
        zoomLevel: 16
        connectivityMode: Map.OfflineMode
        mapType: pannableMap.satelliteMap ? Map.SatelliteMapDay
                                          : Map.StreetMap

        // The initial coordinate where map opens.
        // Set on Riga center
        center: Coordinate {
            longitude: 24.102287
            latitude: 56.944319
        }

        MapCircle {
            id: mapCircle

            center: Coordinate {
                longitude: 24.102287
                latitude: 56.944319
            }

        }


    }

    MapImage {
        id: mapImage

        coordinate: mapCircle.center
        source: "./images/hereicon.png"
        smooth: true
        offset.x: -10
        offset.y: -10
    }

    PinchArea {
        id: pincharea

        property double oldZoomLevel

        function calcZoomDelta(zoom, percent) {
            var zoomLevel = zoom + Math.log(percent) / Math.log(2);
            zoomLevel = Math.round(zoomLevel);
            if (zoomLevel > map.maximumZoomLevel) {
                zoomLevel = map.maximumZoomLevel;
            }
            else if (zoomLevel < map.minimumZoomLevel) {
                zoomLevel = map.minimumZoomLevel;
            }

            return zoomLevel;
        }

        anchors.fill: parent
        onPinchStarted: {
            oldZoomLevel = map.zoomLevel;
        }

        onPinchUpdated: {
            if (oldZoomLevel >= map.maximumZoomLevel &&
                    pinch.scale > 1.0) {
                // Don't allow the user to zoom more than the map supports
                return;
            }
            else if (oldZoomLevel <= map.minimumZoomLevel &&
                     pinch.scale < 1.0) {
                // Don't allow the user to zoom less than the map supports
                return;
            }

            map.scale = pinch.scale;
        }

        onPinchFinished: {
            map.zoomLevel = calcZoomDelta(oldZoomLevel, pinch.scale);
            map.scale = 1;
        }
    }

    MouseArea {
        id: panMouseArea

        property bool mouseDown: false
        property int lastX: -1
        property int lastY: -1

        anchors.fill: parent

        onPressed: {
            panAnimation.stop();
            mouseDown = true;
            lastX = mouse.x;
            lastY = mouse.y;
        }

        onReleased: {
            mouseDown = false;
            lastX = -1;
            lastY = -1;
        }

        onPositionChanged: {
            if (mouseDown) {
                var dx = mouse.x - lastX;
                var dy = mouse.y - lastY;
                map.pan(-dx, -dy);
                lastX = mouse.x;
                lastY = mouse.y;
            }
        }

        onCanceled: {
            mouseDown = false;
        }
    }

    ParallelAnimation {
        id: hereCenterAnimation

        property real latitude
        property real longitude
        property real accuracyInMeters

        PropertyAnimation {
            target: mapCircle
            property: "center.latitude"
            to: hereCenterAnimation.latitude
            easing.type: Easing.InOutCubic
            duration: 1000
        }

        PropertyAnimation {
            target: mapCircle
            property: "center.longitude"
            to: hereCenterAnimation.longitude
            easing.type: Easing.InOutCubic
            duration: 1000
        }

        PropertyAnimation {
            target: mapCircle
            property: "radius"
            to: hereCenterAnimation.accuracyInMeters
            easing.type: Easing.InOutCubic
            duration: 1000
        }
    }

    ParallelAnimation {
        id: panAnimation

        property real latitude
        property real longitude

        PropertyAnimation {
            target: map
            property: "center.latitude"
            to: panAnimation.latitude
            easing.type: Easing.InOutCubic
            duration: 1000
        }

        PropertyAnimation {
            target: map
            property: "center.longitude"
            to: panAnimation.longitude
            easing.type: Easing.InOutCubic
            duration: 1000
        }
    }


}
